
#ifndef _MANAGEMENT_BROKER_
#define _MANAGEMENT_BROKER_

//
// Licensed to the Apache Software Foundation (ASF) under one
// or more contributor license agreements.  See the NOTICE file
// distributed with this work for additional information
// regarding copyright ownership.  The ASF licenses this file
// to you under the Apache License, Version 2.0 (the
// "License"); you may not use this file except in compliance
// with the License.  You may obtain a copy of the License at
// 
//   http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing,
// software distributed under the License is distributed on an
// "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
// KIND, either express or implied.  See the License for the
// specific language governing permissions and limitations
// under the License.
//

// This source file was created by a code generator.
// Please do not edit.

#include "qpid/management/ManagementObject.h"
#include "qpid/framing/FieldTable.h"
#include "qpid/framing/Uuid.h"

namespace qpid {
namespace management {

class Broker : public ManagementObject
{
  private:

    static std::string packageName;
    static std::string className;
    static uint8_t     md5Sum[16];

    // Configuration Elements
    uint64_t systemRef;
    uint16_t port;
    uint16_t workerThreads;
    uint16_t maxConns;
    uint16_t connBacklog;
    uint32_t stagingThreshold;
    uint16_t mgmtPubInterval;
    std::string clusterName;
    std::string version;
    uint8_t dataDirEnabled;
    std::string dataDir;

    // Instrumentation Elements

    // Private Methods
    static void writeSchema   (qpid::framing::Buffer& buf);
    void writeConfig          (qpid::framing::Buffer& buf);
    void writeInstrumentation (qpid::framing::Buffer& buf,
                               bool skipHeaders = false);
    void doMethod             (std::string            methodName,
                               qpid::framing::Buffer& inBuf,
                               qpid::framing::Buffer& outBuf);
    writeSchemaCall_t getWriteSchemaCall (void) { return writeSchema; }

    // Stub for getInstChanged.  There are no inst elements
    bool getInstChanged (void) { return false; }

  public:

    friend class PackageQpid;
    typedef boost::shared_ptr<Broker> shared_ptr;

    Broker (Manageable* coreObject, Manageable* _parent, uint16_t _port);
    ~Broker (void);

    std::string getPackageName (void) { return packageName; }
    std::string getClassName   (void) { return className; }
    uint8_t*    getMd5Sum      (void) { return md5Sum; }

    // Method IDs
    static const uint32_t METHOD_JOINCLUSTER = 1;
    static const uint32_t METHOD_LEAVECLUSTER = 2;
    static const uint32_t METHOD_ECHO = 3;
    static const uint32_t METHOD_CONNECT = 4;

    // Accessor Methods
    inline void set_workerThreads (uint16_t val){
        sys::RWlock::ScopedWlock writeLock (accessLock);
        workerThreads = val;
        configChanged = true;
    }
    inline void set_maxConns (uint16_t val){
        sys::RWlock::ScopedWlock writeLock (accessLock);
        maxConns = val;
        configChanged = true;
    }
    inline void set_connBacklog (uint16_t val){
        sys::RWlock::ScopedWlock writeLock (accessLock);
        connBacklog = val;
        configChanged = true;
    }
    inline void set_stagingThreshold (uint32_t val){
        sys::RWlock::ScopedWlock writeLock (accessLock);
        stagingThreshold = val;
        configChanged = true;
    }
    inline void set_mgmtPubInterval (uint16_t val){
        sys::RWlock::ScopedWlock writeLock (accessLock);
        mgmtPubInterval = val;
        configChanged = true;
    }
    inline void set_clusterName (std::string val){
        sys::RWlock::ScopedWlock writeLock (accessLock);
        clusterName = val;
        configChanged = true;
    }
    inline void set_version (std::string val){
        sys::RWlock::ScopedWlock writeLock (accessLock);
        version = val;
        configChanged = true;
    }
    inline void set_dataDirEnabled (uint8_t val){
        sys::RWlock::ScopedWlock writeLock (accessLock);
        dataDirEnabled = val;
        configChanged = true;
    }
    inline void set_dataDir (std::string val){
        sys::RWlock::ScopedWlock writeLock (accessLock);
        dataDir = val;
        configChanged = true;
    }

};

}}
            

#endif  /*!_MANAGEMENT_BROKER_*/
