
//
// Licensed to the Apache Software Foundation (ASF) under one
// or more contributor license agreements.  See the NOTICE file
// distributed with this work for additional information
// regarding copyright ownership.  The ASF licenses this file
// to you under the Apache License, Version 2.0 (the
// "License"); you may not use this file except in compliance
// with the License.  You may obtain a copy of the License at
// 
//   http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing,
// software distributed under the License is distributed on an
// "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
// KIND, either express or implied.  See the License for the
// specific language governing permissions and limitations
// under the License.
//

// This source file was created by a code generator.
// Please do not edit.

#include "qpid/log/Statement.h"
#include "qpid/framing/FieldTable.h"
#include "qpid/management/Manageable.h" 
#include "Consumer.h"


using namespace qpid::management;
using namespace qpid::sys;
using namespace qpid::framing;
using           std::string;

string  Consumer::packageName  = string ("qpid");
string  Consumer::className    = string ("consumer");
uint8_t Consumer::md5Sum[16]   =
    {0xc3,0x3b,0x95,0x2a,0x52,0xf3,0x60,0x2f,0x96,0x6e,0xd4,0x24,0xce,0x95,0x7b,0x5e};

Consumer::Consumer (Manageable* _core, uint64_t _destinationRef, uint64_t _queueRef) :
    ManagementObject(_core)
    ,destinationRef(_destinationRef),queueRef(_queueRef)
{
    
    msgsConsumed = 0;
    bytesConsumed = 0;
    unackedMessages = 0;
    unackedMessagesHigh = 0;
    unackedMessagesLow  = 0;

}

Consumer::~Consumer () {}

namespace {
    const string NAME("name");
    const string TYPE("type");
    const string ACCESS("access");
    const string INDEX("index");
    const string UNIT("unit");
    const string MIN("min");
    const string MAX("max");
    const string MAXLEN("maxlen");
    const string DESC("desc");
    const string ARGCOUNT("argCount");
    const string ARGS("args");
    const string DIR("dir");
    const string DEFAULT("default");
}

void Consumer::writeSchema (Buffer& buf)
{
    FieldTable ft;

    // Schema class header:
    buf.putShortString (packageName); // Package Name
    buf.putShortString (className);   // Class Name
    buf.putBin128      (md5Sum);      // Schema Hash
    buf.putShort       (2); // Config Element Count
    buf.putShort       (5); // Inst Element Count
    buf.putShort       (1); // Method Count
    buf.putShort       (0); // Event Count

    // Config Elements
    ft = FieldTable ();
    ft.setString (NAME,   "destinationRef");
    ft.setInt    (TYPE,   TYPE_REF);
    ft.setInt    (ACCESS, ACCESS_RC);
    ft.setInt    (INDEX,  1);
    buf.put (ft);

    ft = FieldTable ();
    ft.setString (NAME,   "queueRef");
    ft.setInt    (TYPE,   TYPE_REF);
    ft.setInt    (ACCESS, ACCESS_RC);
    ft.setInt    (INDEX,  1);
    buf.put (ft);


    // Inst Elements
    ft = FieldTable ();
    ft.setString (NAME,   "msgsConsumed");
    ft.setInt    (TYPE,   TYPE_U64);
    buf.put (ft);

    ft = FieldTable ();
    ft.setString (NAME,   "bytesConsumed");
    ft.setInt    (TYPE,   TYPE_U64);
    buf.put (ft);

    ft = FieldTable ();
    ft.setString (NAME,   "unackedMessages");
    ft.setInt    (TYPE,   TYPE_U32);
    ft.setString (DESC,   "Messages consumed but not yet acked");
    buf.put (ft);

    ft = FieldTable ();
    ft.setString (NAME,   "unackedMessagesHigh");
    ft.setInt    (TYPE,   TYPE_U32);
    ft.setString (DESC,   "Messages consumed but not yet acked (High)");
    buf.put (ft);

    ft = FieldTable ();
    ft.setString (NAME,   "unackedMessagesLow");
    ft.setInt    (TYPE,   TYPE_U32);
    ft.setString (DESC,   "Messages consumed but not yet acked (Low)");
    buf.put (ft);


    // Methods
    ft = FieldTable ();
    ft.setString (NAME,     "close");
    ft.setInt    (ARGCOUNT, 0);
    buf.put (ft);


    // Events

}

void Consumer::writeConfig (Buffer& buf)
{
    sys::RWlock::ScopedRlock readLock (accessLock);
    configChanged = false;

    writeTimestamps (buf);
    buf.putLongLong (destinationRef);
    buf.putLongLong (queueRef);

}

void Consumer::writeInstrumentation (Buffer& buf, bool skipHeaders)
{
    sys::RWlock::ScopedWlock writeLock (accessLock);
    instChanged = false;

    if (!skipHeaders)
        writeTimestamps (buf);
    buf.putLongLong (msgsConsumed);
    buf.putLongLong (bytesConsumed);
    buf.putLong (unackedMessages);
    buf.putLong (unackedMessagesHigh);
    buf.putLong (unackedMessagesLow);


    // Maintenance of hi-lo statistics
    unackedMessagesHigh = unackedMessages;
    unackedMessagesLow  = unackedMessages;

}

void Consumer::doMethod (string methodName, Buffer&, Buffer& outBuf)
{
    Manageable::status_t status = Manageable::STATUS_UNKNOWN_METHOD;

    if (methodName == "close")
    {
        ArgsNone ioArgs;
        status = coreObject->ManagementMethod (METHOD_CLOSE, ioArgs);
        outBuf.putLong        (status);
        outBuf.putShortString (Manageable::StatusText (status));
        return;
    }

    outBuf.putLong        (status);
    outBuf.putShortString (Manageable::StatusText (status));
}

