
#ifndef _MANAGEMENT_CONSUMER_
#define _MANAGEMENT_CONSUMER_

//
// Licensed to the Apache Software Foundation (ASF) under one
// or more contributor license agreements.  See the NOTICE file
// distributed with this work for additional information
// regarding copyright ownership.  The ASF licenses this file
// to you under the Apache License, Version 2.0 (the
// "License"); you may not use this file except in compliance
// with the License.  You may obtain a copy of the License at
// 
//   http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing,
// software distributed under the License is distributed on an
// "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
// KIND, either express or implied.  See the License for the
// specific language governing permissions and limitations
// under the License.
//

// This source file was created by a code generator.
// Please do not edit.

#include "qpid/management/ManagementObject.h"
#include "qpid/framing/FieldTable.h"
#include "qpid/framing/Uuid.h"

namespace qpid {
namespace management {

class Consumer : public ManagementObject
{
  private:

    static std::string packageName;
    static std::string className;
    static uint8_t     md5Sum[16];

    // Configuration Elements
    uint64_t destinationRef;
    uint64_t queueRef;

    // Instrumentation Elements
    uint64_t  msgsConsumed;
    uint64_t  bytesConsumed;
    uint32_t  unackedMessages;
    uint32_t  unackedMessagesHigh;
    uint32_t  unackedMessagesLow;

    // Private Methods
    static void writeSchema   (qpid::framing::Buffer& buf);
    void writeConfig          (qpid::framing::Buffer& buf);
    void writeInstrumentation (qpid::framing::Buffer& buf,
                               bool skipHeaders = false);
    void doMethod             (std::string            methodName,
                               qpid::framing::Buffer& inBuf,
                               qpid::framing::Buffer& outBuf);
    writeSchemaCall_t getWriteSchemaCall (void) { return writeSchema; }


  public:

    friend class PackageQpid;
    typedef boost::shared_ptr<Consumer> shared_ptr;

    Consumer (Manageable* coreObject, uint64_t _destinationRef, uint64_t _queueRef);
    ~Consumer (void);

    std::string getPackageName (void) { return packageName; }
    std::string getClassName   (void) { return className; }
    uint8_t*    getMd5Sum      (void) { return md5Sum; }

    // Method IDs
    static const uint32_t METHOD_CLOSE = 1;

    // Accessor Methods
    inline void inc_msgsConsumed (uint64_t by = 1){
        sys::RWlock::ScopedWlock writeLock (accessLock);
        msgsConsumed += by;
        instChanged = true;
    }
    inline void dec_msgsConsumed (uint64_t by = 1){
        sys::RWlock::ScopedWlock writeLock (accessLock);
        msgsConsumed -= by;
        instChanged = true;
    }
    inline void set_msgsConsumed (uint64_t val){
        sys::RWlock::ScopedWlock writeLock (accessLock);
        msgsConsumed = val;
        instChanged = true;
    }
    inline void inc_bytesConsumed (uint64_t by = 1){
        sys::RWlock::ScopedWlock writeLock (accessLock);
        bytesConsumed += by;
        instChanged = true;
    }
    inline void dec_bytesConsumed (uint64_t by = 1){
        sys::RWlock::ScopedWlock writeLock (accessLock);
        bytesConsumed -= by;
        instChanged = true;
    }
    inline void set_bytesConsumed (uint64_t val){
        sys::RWlock::ScopedWlock writeLock (accessLock);
        bytesConsumed = val;
        instChanged = true;
    }
    inline void inc_unackedMessages (uint32_t by = 1){
        sys::RWlock::ScopedWlock writeLock (accessLock);
        unackedMessages += by;
        if (unackedMessagesHigh < unackedMessages)
            unackedMessagesHigh = unackedMessages;
        instChanged = true;
    }
    inline void dec_unackedMessages (uint32_t by = 1){
        sys::RWlock::ScopedWlock writeLock (accessLock);
        unackedMessages -= by;
        if (unackedMessagesLow > unackedMessages)
            unackedMessagesLow = unackedMessages;
        instChanged = true;
    }
    inline void set_unackedMessages (uint32_t val){
        sys::RWlock::ScopedWlock writeLock (accessLock);
        unackedMessages = val;
        if (unackedMessagesLow  > val)
            unackedMessagesLow  = val;
        if (unackedMessagesHigh < val)
            unackedMessagesHigh = val;
        instChanged = true;
    }

};

}}
            

#endif  /*!_MANAGEMENT_CONSUMER_*/
