
//
// Licensed to the Apache Software Foundation (ASF) under one
// or more contributor license agreements.  See the NOTICE file
// distributed with this work for additional information
// regarding copyright ownership.  The ASF licenses this file
// to you under the Apache License, Version 2.0 (the
// "License"); you may not use this file except in compliance
// with the License.  You may obtain a copy of the License at
// 
//   http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing,
// software distributed under the License is distributed on an
// "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
// KIND, either express or implied.  See the License for the
// specific language governing permissions and limitations
// under the License.
//

// This source file was created by a code generator.
// Please do not edit.

#include "qpid/log/Statement.h"
#include "qpid/framing/FieldTable.h"
#include "qpid/management/Manageable.h" 
#include "Destination.h"
#include "qpid/management/ArgsDestinationThrottle.h"


using namespace qpid::management;
using namespace qpid::sys;
using namespace qpid::framing;
using           std::string;

string  Destination::packageName  = string ("qpid");
string  Destination::className    = string ("destination");
uint8_t Destination::md5Sum[16]   =
    {0x14,0x59,0x82,0x8,0x33,0xb9,0x45,0x2d,0x15,0x57,0xfd,0x64,0xf,0xec,0xee,0xbe};

Destination::Destination (Manageable* _core, Manageable* _parent, std::string _name) :
    ManagementObject(_core)
    ,name(_name)
{
    sessionRef = _parent->GetManagementObject ()->getObjectId ();
    flowMode = 0;
    maxMsgCredits = 0;
    maxByteCredits = 0;
    msgCredits = 0;
    byteCredits = 0;

}

Destination::~Destination () {}

namespace {
    const string NAME("name");
    const string TYPE("type");
    const string ACCESS("access");
    const string INDEX("index");
    const string UNIT("unit");
    const string MIN("min");
    const string MAX("max");
    const string MAXLEN("maxlen");
    const string DESC("desc");
    const string ARGCOUNT("argCount");
    const string ARGS("args");
    const string DIR("dir");
    const string DEFAULT("default");
}

void Destination::writeSchema (Buffer& buf)
{
    FieldTable ft;

    // Schema class header:
    buf.putShortString (packageName); // Package Name
    buf.putShortString (className);   // Class Name
    buf.putBin128      (md5Sum);      // Schema Hash
    buf.putShort       (2); // Config Element Count
    buf.putShort       (5); // Inst Element Count
    buf.putShort       (3); // Method Count
    buf.putShort       (0); // Event Count

    // Config Elements
    ft = FieldTable ();
    ft.setString (NAME,   "sessionRef");
    ft.setInt    (TYPE,   TYPE_REF);
    ft.setInt    (ACCESS, ACCESS_RC);
    ft.setInt    (INDEX,  1);
    buf.put (ft);

    ft = FieldTable ();
    ft.setString (NAME,   "name");
    ft.setInt    (TYPE,   TYPE_SSTR);
    ft.setInt    (ACCESS, ACCESS_RC);
    ft.setInt    (INDEX,  1);
    buf.put (ft);


    // Inst Elements
    ft = FieldTable ();
    ft.setString (NAME,   "flowMode");
    ft.setInt    (TYPE,   TYPE_U8);
    buf.put (ft);

    ft = FieldTable ();
    ft.setString (NAME,   "maxMsgCredits");
    ft.setInt    (TYPE,   TYPE_U32);
    buf.put (ft);

    ft = FieldTable ();
    ft.setString (NAME,   "maxByteCredits");
    ft.setInt    (TYPE,   TYPE_U32);
    buf.put (ft);

    ft = FieldTable ();
    ft.setString (NAME,   "msgCredits");
    ft.setInt    (TYPE,   TYPE_U32);
    buf.put (ft);

    ft = FieldTable ();
    ft.setString (NAME,   "byteCredits");
    ft.setInt    (TYPE,   TYPE_U32);
    buf.put (ft);


    // Methods
    ft = FieldTable ();
    ft.setString (NAME,     "throttle");
    ft.setInt    (ARGCOUNT, 1);
    ft.setString (DESC,     "Apply extra rate limiting to destination: 0 = Normal, 10 = Maximum");
    buf.put (ft);

    ft = FieldTable ();
    ft.setString (NAME,    "strength");
    ft.setInt    (TYPE,    TYPE_U8);
    ft.setString (DIR,     "I");
    ft.setInt    (MIN,     0);
    ft.setInt    (MAX,     10);
    buf.put (ft);

    ft = FieldTable ();
    ft.setString (NAME,     "stop");
    ft.setInt    (ARGCOUNT, 0);
    buf.put (ft);

    ft = FieldTable ();
    ft.setString (NAME,     "start");
    ft.setInt    (ARGCOUNT, 0);
    buf.put (ft);


    // Events

}

void Destination::writeConfig (Buffer& buf)
{
    sys::RWlock::ScopedRlock readLock (accessLock);
    configChanged = false;

    writeTimestamps (buf);
    buf.putLongLong (sessionRef);
    buf.putShortString (name);

}

void Destination::writeInstrumentation (Buffer& buf, bool skipHeaders)
{
    sys::RWlock::ScopedWlock writeLock (accessLock);
    instChanged = false;

    if (!skipHeaders)
        writeTimestamps (buf);
    buf.putOctet (flowMode);
    buf.putLong (maxMsgCredits);
    buf.putLong (maxByteCredits);
    buf.putLong (msgCredits);
    buf.putLong (byteCredits);


    // Maintenance of hi-lo statistics

}

void Destination::doMethod (string methodName, Buffer& inBuf, Buffer& outBuf)
{
    Manageable::status_t status = Manageable::STATUS_UNKNOWN_METHOD;

    if (methodName == "throttle")
    {
        ArgsDestinationThrottle ioArgs;
        ioArgs.i_strength = inBuf.getOctet ();
        status = coreObject->ManagementMethod (METHOD_THROTTLE, ioArgs);
        outBuf.putLong        (status);
        outBuf.putShortString (Manageable::StatusText (status));
        return;
    }

    if (methodName == "stop")
    {
        ArgsNone ioArgs;
        status = coreObject->ManagementMethod (METHOD_STOP, ioArgs);
        outBuf.putLong        (status);
        outBuf.putShortString (Manageable::StatusText (status));
        return;
    }

    if (methodName == "start")
    {
        ArgsNone ioArgs;
        status = coreObject->ManagementMethod (METHOD_START, ioArgs);
        outBuf.putLong        (status);
        outBuf.putShortString (Manageable::StatusText (status));
        return;
    }

    outBuf.putLong        (status);
    outBuf.putShortString (Manageable::StatusText (status));
}

