
//
// Licensed to the Apache Software Foundation (ASF) under one
// or more contributor license agreements.  See the NOTICE file
// distributed with this work for additional information
// regarding copyright ownership.  The ASF licenses this file
// to you under the Apache License, Version 2.0 (the
// "License"); you may not use this file except in compliance
// with the License.  You may obtain a copy of the License at
// 
//   http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing,
// software distributed under the License is distributed on an
// "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
// KIND, either express or implied.  See the License for the
// specific language governing permissions and limitations
// under the License.
//

// This source file was created by a code generator.
// Please do not edit.

#include "qpid/log/Statement.h"
#include "qpid/framing/FieldTable.h"
#include "qpid/management/Manageable.h" 
#include "Exchange.h"


using namespace qpid::management;
using namespace qpid::sys;
using namespace qpid::framing;
using           std::string;

string  Exchange::packageName  = string ("qpid");
string  Exchange::className    = string ("exchange");
uint8_t Exchange::md5Sum[16]   =
    {0xb4,0xcc,0x9e,0xec,0x4f,0x7e,0x25,0x34,0xbc,0xf,0x19,0xe2,0xd,0xc6,0xc3,0xba};

Exchange::Exchange (Manageable* _core, Manageable* _parent, std::string _name, uint8_t _durable) :
    ManagementObject(_core)
    ,name(_name),durable(_durable)
{
    vhostRef = _parent->GetManagementObject ()->getObjectId ();
    producers = 0;
    producersHigh = 0;
    producersLow  = 0;
    bindings = 0;
    bindingsHigh = 0;
    bindingsLow  = 0;
    msgReceives = 0;
    msgDrops = 0;
    msgRoutes = 0;
    byteReceives = 0;
    byteDrops = 0;
    byteRoutes = 0;

}

Exchange::~Exchange () {}

namespace {
    const string NAME("name");
    const string TYPE("type");
    const string ACCESS("access");
    const string INDEX("index");
    const string UNIT("unit");
    const string MIN("min");
    const string MAX("max");
    const string MAXLEN("maxlen");
    const string DESC("desc");
    const string ARGCOUNT("argCount");
    const string ARGS("args");
    const string DIR("dir");
    const string DEFAULT("default");
}

void Exchange::writeSchema (Buffer& buf)
{
    FieldTable ft;

    // Schema class header:
    buf.putShortString (packageName); // Package Name
    buf.putShortString (className);   // Class Name
    buf.putBin128      (md5Sum);      // Schema Hash
    buf.putShort       (4); // Config Element Count
    buf.putShort       (12); // Inst Element Count
    buf.putShort       (0); // Method Count
    buf.putShort       (0); // Event Count

    // Config Elements
    ft = FieldTable ();
    ft.setString (NAME,   "vhostRef");
    ft.setInt    (TYPE,   TYPE_REF);
    ft.setInt    (ACCESS, ACCESS_RC);
    ft.setInt    (INDEX,  1);
    buf.put (ft);

    ft = FieldTable ();
    ft.setString (NAME,   "name");
    ft.setInt    (TYPE,   TYPE_SSTR);
    ft.setInt    (ACCESS, ACCESS_RC);
    ft.setInt    (INDEX,  1);
    buf.put (ft);

    ft = FieldTable ();
    ft.setString (NAME,   "type");
    ft.setInt    (TYPE,   TYPE_SSTR);
    ft.setInt    (ACCESS, ACCESS_RO);
    ft.setInt    (INDEX,  0);
    buf.put (ft);

    ft = FieldTable ();
    ft.setString (NAME,   "durable");
    ft.setInt    (TYPE,   TYPE_BOOL);
    ft.setInt    (ACCESS, ACCESS_RC);
    ft.setInt    (INDEX,  0);
    buf.put (ft);


    // Inst Elements
    ft = FieldTable ();
    ft.setString (NAME,   "producers");
    ft.setInt    (TYPE,   TYPE_U32);
    ft.setString (DESC,   "Current producers on exchange");
    buf.put (ft);

    ft = FieldTable ();
    ft.setString (NAME,   "producersHigh");
    ft.setInt    (TYPE,   TYPE_U32);
    ft.setString (DESC,   "Current producers on exchange (High)");
    buf.put (ft);

    ft = FieldTable ();
    ft.setString (NAME,   "producersLow");
    ft.setInt    (TYPE,   TYPE_U32);
    ft.setString (DESC,   "Current producers on exchange (Low)");
    buf.put (ft);

    ft = FieldTable ();
    ft.setString (NAME,   "bindings");
    ft.setInt    (TYPE,   TYPE_U32);
    ft.setString (DESC,   "Current bindings");
    buf.put (ft);

    ft = FieldTable ();
    ft.setString (NAME,   "bindingsHigh");
    ft.setInt    (TYPE,   TYPE_U32);
    ft.setString (DESC,   "Current bindings (High)");
    buf.put (ft);

    ft = FieldTable ();
    ft.setString (NAME,   "bindingsLow");
    ft.setInt    (TYPE,   TYPE_U32);
    ft.setString (DESC,   "Current bindings (Low)");
    buf.put (ft);

    ft = FieldTable ();
    ft.setString (NAME,   "msgReceives");
    ft.setInt    (TYPE,   TYPE_U64);
    ft.setString (DESC,   "Total messages received");
    buf.put (ft);

    ft = FieldTable ();
    ft.setString (NAME,   "msgDrops");
    ft.setInt    (TYPE,   TYPE_U64);
    ft.setString (DESC,   "Total messages dropped (no matching key)");
    buf.put (ft);

    ft = FieldTable ();
    ft.setString (NAME,   "msgRoutes");
    ft.setInt    (TYPE,   TYPE_U64);
    ft.setString (DESC,   "Total routed messages");
    buf.put (ft);

    ft = FieldTable ();
    ft.setString (NAME,   "byteReceives");
    ft.setInt    (TYPE,   TYPE_U64);
    ft.setString (DESC,   "Total bytes received");
    buf.put (ft);

    ft = FieldTable ();
    ft.setString (NAME,   "byteDrops");
    ft.setInt    (TYPE,   TYPE_U64);
    ft.setString (DESC,   "Total bytes dropped (no matching key)");
    buf.put (ft);

    ft = FieldTable ();
    ft.setString (NAME,   "byteRoutes");
    ft.setInt    (TYPE,   TYPE_U64);
    ft.setString (DESC,   "Total routed bytes");
    buf.put (ft);


    // Methods

    // Events

}

void Exchange::writeConfig (Buffer& buf)
{
    sys::RWlock::ScopedRlock readLock (accessLock);
    configChanged = false;

    writeTimestamps (buf);
    buf.putLongLong (vhostRef);
    buf.putShortString (name);
    buf.putShortString (type);
    buf.putOctet (durable?1:0);

}

void Exchange::writeInstrumentation (Buffer& buf, bool skipHeaders)
{
    sys::RWlock::ScopedWlock writeLock (accessLock);
    instChanged = false;

    if (!skipHeaders)
        writeTimestamps (buf);
    buf.putLong (producers);
    buf.putLong (producersHigh);
    buf.putLong (producersLow);
    buf.putLong (bindings);
    buf.putLong (bindingsHigh);
    buf.putLong (bindingsLow);
    buf.putLongLong (msgReceives);
    buf.putLongLong (msgDrops);
    buf.putLongLong (msgRoutes);
    buf.putLongLong (byteReceives);
    buf.putLongLong (byteDrops);
    buf.putLongLong (byteRoutes);


    // Maintenance of hi-lo statistics
    producersHigh = producers;
    producersLow  = producers;
    bindingsHigh = bindings;
    bindingsLow  = bindings;

}

void Exchange::doMethod (string, Buffer&, Buffer& outBuf)
{
    Manageable::status_t status = Manageable::STATUS_UNKNOWN_METHOD;

    outBuf.putLong        (status);
    outBuf.putShortString (Manageable::StatusText (status));
}

