
#ifndef _MANAGEMENT_EXCHANGE_
#define _MANAGEMENT_EXCHANGE_

//
// Licensed to the Apache Software Foundation (ASF) under one
// or more contributor license agreements.  See the NOTICE file
// distributed with this work for additional information
// regarding copyright ownership.  The ASF licenses this file
// to you under the Apache License, Version 2.0 (the
// "License"); you may not use this file except in compliance
// with the License.  You may obtain a copy of the License at
// 
//   http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing,
// software distributed under the License is distributed on an
// "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
// KIND, either express or implied.  See the License for the
// specific language governing permissions and limitations
// under the License.
//

// This source file was created by a code generator.
// Please do not edit.

#include "qpid/management/ManagementObject.h"
#include "qpid/framing/FieldTable.h"
#include "qpid/framing/Uuid.h"

namespace qpid {
namespace management {

class Exchange : public ManagementObject
{
  private:

    static std::string packageName;
    static std::string className;
    static uint8_t     md5Sum[16];

    // Configuration Elements
    uint64_t vhostRef;
    std::string name;
    std::string type;
    uint8_t durable;

    // Instrumentation Elements
    uint32_t  producers;
    uint32_t  producersHigh;
    uint32_t  producersLow;
    uint32_t  bindings;
    uint32_t  bindingsHigh;
    uint32_t  bindingsLow;
    uint64_t  msgReceives;
    uint64_t  msgDrops;
    uint64_t  msgRoutes;
    uint64_t  byteReceives;
    uint64_t  byteDrops;
    uint64_t  byteRoutes;

    // Private Methods
    static void writeSchema   (qpid::framing::Buffer& buf);
    void writeConfig          (qpid::framing::Buffer& buf);
    void writeInstrumentation (qpid::framing::Buffer& buf,
                               bool skipHeaders = false);
    void doMethod             (std::string            methodName,
                               qpid::framing::Buffer& inBuf,
                               qpid::framing::Buffer& outBuf);
    writeSchemaCall_t getWriteSchemaCall (void) { return writeSchema; }


  public:

    friend class PackageQpid;
    typedef boost::shared_ptr<Exchange> shared_ptr;

    Exchange (Manageable* coreObject, Manageable* _parent, std::string _name, uint8_t _durable);
    ~Exchange (void);

    std::string getPackageName (void) { return packageName; }
    std::string getClassName   (void) { return className; }
    uint8_t*    getMd5Sum      (void) { return md5Sum; }

    // Method IDs

    // Accessor Methods
    inline void set_type (std::string val){
        sys::RWlock::ScopedWlock writeLock (accessLock);
        type = val;
        configChanged = true;
    }
    inline void inc_producers (uint32_t by = 1){
        sys::RWlock::ScopedWlock writeLock (accessLock);
        producers += by;
        if (producersHigh < producers)
            producersHigh = producers;
        instChanged = true;
    }
    inline void dec_producers (uint32_t by = 1){
        sys::RWlock::ScopedWlock writeLock (accessLock);
        producers -= by;
        if (producersLow > producers)
            producersLow = producers;
        instChanged = true;
    }
    inline void set_producers (uint32_t val){
        sys::RWlock::ScopedWlock writeLock (accessLock);
        producers = val;
        if (producersLow  > val)
            producersLow  = val;
        if (producersHigh < val)
            producersHigh = val;
        instChanged = true;
    }
    inline void inc_bindings (uint32_t by = 1){
        sys::RWlock::ScopedWlock writeLock (accessLock);
        bindings += by;
        if (bindingsHigh < bindings)
            bindingsHigh = bindings;
        instChanged = true;
    }
    inline void dec_bindings (uint32_t by = 1){
        sys::RWlock::ScopedWlock writeLock (accessLock);
        bindings -= by;
        if (bindingsLow > bindings)
            bindingsLow = bindings;
        instChanged = true;
    }
    inline void set_bindings (uint32_t val){
        sys::RWlock::ScopedWlock writeLock (accessLock);
        bindings = val;
        if (bindingsLow  > val)
            bindingsLow  = val;
        if (bindingsHigh < val)
            bindingsHigh = val;
        instChanged = true;
    }
    inline void inc_msgReceives (uint64_t by = 1){
        sys::RWlock::ScopedWlock writeLock (accessLock);
        msgReceives += by;
        instChanged = true;
    }
    inline void dec_msgReceives (uint64_t by = 1){
        sys::RWlock::ScopedWlock writeLock (accessLock);
        msgReceives -= by;
        instChanged = true;
    }
    inline void set_msgReceives (uint64_t val){
        sys::RWlock::ScopedWlock writeLock (accessLock);
        msgReceives = val;
        instChanged = true;
    }
    inline void inc_msgDrops (uint64_t by = 1){
        sys::RWlock::ScopedWlock writeLock (accessLock);
        msgDrops += by;
        instChanged = true;
    }
    inline void dec_msgDrops (uint64_t by = 1){
        sys::RWlock::ScopedWlock writeLock (accessLock);
        msgDrops -= by;
        instChanged = true;
    }
    inline void set_msgDrops (uint64_t val){
        sys::RWlock::ScopedWlock writeLock (accessLock);
        msgDrops = val;
        instChanged = true;
    }
    inline void inc_msgRoutes (uint64_t by = 1){
        sys::RWlock::ScopedWlock writeLock (accessLock);
        msgRoutes += by;
        instChanged = true;
    }
    inline void dec_msgRoutes (uint64_t by = 1){
        sys::RWlock::ScopedWlock writeLock (accessLock);
        msgRoutes -= by;
        instChanged = true;
    }
    inline void set_msgRoutes (uint64_t val){
        sys::RWlock::ScopedWlock writeLock (accessLock);
        msgRoutes = val;
        instChanged = true;
    }
    inline void inc_byteReceives (uint64_t by = 1){
        sys::RWlock::ScopedWlock writeLock (accessLock);
        byteReceives += by;
        instChanged = true;
    }
    inline void dec_byteReceives (uint64_t by = 1){
        sys::RWlock::ScopedWlock writeLock (accessLock);
        byteReceives -= by;
        instChanged = true;
    }
    inline void set_byteReceives (uint64_t val){
        sys::RWlock::ScopedWlock writeLock (accessLock);
        byteReceives = val;
        instChanged = true;
    }
    inline void inc_byteDrops (uint64_t by = 1){
        sys::RWlock::ScopedWlock writeLock (accessLock);
        byteDrops += by;
        instChanged = true;
    }
    inline void dec_byteDrops (uint64_t by = 1){
        sys::RWlock::ScopedWlock writeLock (accessLock);
        byteDrops -= by;
        instChanged = true;
    }
    inline void set_byteDrops (uint64_t val){
        sys::RWlock::ScopedWlock writeLock (accessLock);
        byteDrops = val;
        instChanged = true;
    }
    inline void inc_byteRoutes (uint64_t by = 1){
        sys::RWlock::ScopedWlock writeLock (accessLock);
        byteRoutes += by;
        instChanged = true;
    }
    inline void dec_byteRoutes (uint64_t by = 1){
        sys::RWlock::ScopedWlock writeLock (accessLock);
        byteRoutes -= by;
        instChanged = true;
    }
    inline void set_byteRoutes (uint64_t val){
        sys::RWlock::ScopedWlock writeLock (accessLock);
        byteRoutes = val;
        instChanged = true;
    }

};

}}
            

#endif  /*!_MANAGEMENT_EXCHANGE_*/
