
//
// Licensed to the Apache Software Foundation (ASF) under one
// or more contributor license agreements.  See the NOTICE file
// distributed with this work for additional information
// regarding copyright ownership.  The ASF licenses this file
// to you under the Apache License, Version 2.0 (the
// "License"); you may not use this file except in compliance
// with the License.  You may obtain a copy of the License at
// 
//   http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing,
// software distributed under the License is distributed on an
// "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
// KIND, either express or implied.  See the License for the
// specific language governing permissions and limitations
// under the License.
//

// This source file was created by a code generator.
// Please do not edit.

#include "qpid/log/Statement.h"
#include "qpid/framing/FieldTable.h"
#include "qpid/management/Manageable.h" 
#include "Link.h"
#include "qpid/management/ArgsLinkBridge.h"


using namespace qpid::management;
using namespace qpid::sys;
using namespace qpid::framing;
using           std::string;

string  Link::packageName  = string ("qpid");
string  Link::className    = string ("link");
uint8_t Link::md5Sum[16]   =
    {0x89,0x15,0x60,0x8b,0xeb,0x83,0x73,0x25,0x41,0x77,0x86,0x86,0x26,0xf,0x39,0xf8};

Link::Link (Manageable* _core, Manageable* _parent, std::string _host, uint16_t _port, uint8_t _useSsl, uint8_t _durable) :
    ManagementObject(_core)
    ,host(_host),port(_port),useSsl(_useSsl),durable(_durable)
{
    vhostRef = _parent->GetManagementObject ()->getObjectId ();
    state = "";
    lastError = "";

}

Link::~Link () {}

namespace {
    const string NAME("name");
    const string TYPE("type");
    const string ACCESS("access");
    const string INDEX("index");
    const string UNIT("unit");
    const string MIN("min");
    const string MAX("max");
    const string MAXLEN("maxlen");
    const string DESC("desc");
    const string ARGCOUNT("argCount");
    const string ARGS("args");
    const string DIR("dir");
    const string DEFAULT("default");
}

void Link::writeSchema (Buffer& buf)
{
    FieldTable ft;

    // Schema class header:
    buf.putShortString (packageName); // Package Name
    buf.putShortString (className);   // Class Name
    buf.putBin128      (md5Sum);      // Schema Hash
    buf.putShort       (5); // Config Element Count
    buf.putShort       (2); // Inst Element Count
    buf.putShort       (2); // Method Count
    buf.putShort       (0); // Event Count

    // Config Elements
    ft = FieldTable ();
    ft.setString (NAME,   "vhostRef");
    ft.setInt    (TYPE,   TYPE_REF);
    ft.setInt    (ACCESS, ACCESS_RC);
    ft.setInt    (INDEX,  1);
    buf.put (ft);

    ft = FieldTable ();
    ft.setString (NAME,   "host");
    ft.setInt    (TYPE,   TYPE_SSTR);
    ft.setInt    (ACCESS, ACCESS_RC);
    ft.setInt    (INDEX,  1);
    buf.put (ft);

    ft = FieldTable ();
    ft.setString (NAME,   "port");
    ft.setInt    (TYPE,   TYPE_U16);
    ft.setInt    (ACCESS, ACCESS_RC);
    ft.setInt    (INDEX,  1);
    buf.put (ft);

    ft = FieldTable ();
    ft.setString (NAME,   "useSsl");
    ft.setInt    (TYPE,   TYPE_BOOL);
    ft.setInt    (ACCESS, ACCESS_RC);
    ft.setInt    (INDEX,  0);
    buf.put (ft);

    ft = FieldTable ();
    ft.setString (NAME,   "durable");
    ft.setInt    (TYPE,   TYPE_BOOL);
    ft.setInt    (ACCESS, ACCESS_RC);
    ft.setInt    (INDEX,  0);
    buf.put (ft);


    // Inst Elements
    ft = FieldTable ();
    ft.setString (NAME,   "state");
    ft.setInt    (TYPE,   TYPE_SSTR);
    ft.setString (DESC,   "Operational state of the link");
    buf.put (ft);

    ft = FieldTable ();
    ft.setString (NAME,   "lastError");
    ft.setInt    (TYPE,   TYPE_SSTR);
    ft.setString (DESC,   "Reason link is not operational");
    buf.put (ft);


    // Methods
    ft = FieldTable ();
    ft.setString (NAME,     "close");
    ft.setInt    (ARGCOUNT, 0);
    buf.put (ft);

    ft = FieldTable ();
    ft.setString (NAME,     "bridge");
    ft.setInt    (ARGCOUNT, 8);
    ft.setString (DESC,     "Bridge messages over the link");
    buf.put (ft);

    ft = FieldTable ();
    ft.setString (NAME,    "durable");
    ft.setInt    (TYPE,    TYPE_BOOL);
    ft.setString (DIR,     "I");
    buf.put (ft);

    ft = FieldTable ();
    ft.setString (NAME,    "src");
    ft.setInt    (TYPE,    TYPE_SSTR);
    ft.setString (DIR,     "I");
    buf.put (ft);

    ft = FieldTable ();
    ft.setString (NAME,    "dest");
    ft.setInt    (TYPE,    TYPE_SSTR);
    ft.setString (DIR,     "I");
    buf.put (ft);

    ft = FieldTable ();
    ft.setString (NAME,    "key");
    ft.setInt    (TYPE,    TYPE_SSTR);
    ft.setString (DIR,     "I");
    ft.setString (DEFAULT, "");
    buf.put (ft);

    ft = FieldTable ();
    ft.setString (NAME,    "tag");
    ft.setInt    (TYPE,    TYPE_SSTR);
    ft.setString (DIR,     "I");
    ft.setString (DEFAULT, "");
    buf.put (ft);

    ft = FieldTable ();
    ft.setString (NAME,    "excludes");
    ft.setInt    (TYPE,    TYPE_SSTR);
    ft.setString (DIR,     "I");
    ft.setString (DEFAULT, "");
    buf.put (ft);

    ft = FieldTable ();
    ft.setString (NAME,    "src_is_queue");
    ft.setInt    (TYPE,    TYPE_BOOL);
    ft.setString (DIR,     "I");
    ft.setString (DEFAULT, "0");
    buf.put (ft);

    ft = FieldTable ();
    ft.setString (NAME,    "src_is_local");
    ft.setInt    (TYPE,    TYPE_BOOL);
    ft.setString (DIR,     "I");
    ft.setString (DEFAULT, "0");
    buf.put (ft);


    // Events

}

void Link::writeConfig (Buffer& buf)
{
    sys::RWlock::ScopedRlock readLock (accessLock);
    configChanged = false;

    writeTimestamps (buf);
    buf.putLongLong (vhostRef);
    buf.putShortString (host);
    buf.putShort (port);
    buf.putOctet (useSsl?1:0);
    buf.putOctet (durable?1:0);

}

void Link::writeInstrumentation (Buffer& buf, bool skipHeaders)
{
    sys::RWlock::ScopedWlock writeLock (accessLock);
    instChanged = false;

    if (!skipHeaders)
        writeTimestamps (buf);
    buf.putShortString (state);
    buf.putShortString (lastError);


    // Maintenance of hi-lo statistics

}

void Link::doMethod (string methodName, Buffer& inBuf, Buffer& outBuf)
{
    Manageable::status_t status = Manageable::STATUS_UNKNOWN_METHOD;

    if (methodName == "close")
    {
        ArgsNone ioArgs;
        status = coreObject->ManagementMethod (METHOD_CLOSE, ioArgs);
        outBuf.putLong        (status);
        outBuf.putShortString (Manageable::StatusText (status));
        return;
    }

    if (methodName == "bridge")
    {
        ArgsLinkBridge ioArgs;
        ioArgs.i_durable = inBuf.getOctet ()==1;
        inBuf.getShortString (ioArgs.i_src);
        inBuf.getShortString (ioArgs.i_dest);
        inBuf.getShortString (ioArgs.i_key);
        inBuf.getShortString (ioArgs.i_tag);
        inBuf.getShortString (ioArgs.i_excludes);
        ioArgs.i_src_is_queue = inBuf.getOctet ()==1;
        ioArgs.i_src_is_local = inBuf.getOctet ()==1;
        status = coreObject->ManagementMethod (METHOD_BRIDGE, ioArgs);
        outBuf.putLong        (status);
        outBuf.putShortString (Manageable::StatusText (status));
        return;
    }

    outBuf.putLong        (status);
    outBuf.putShortString (Manageable::StatusText (status));
}

