
#ifndef _MANAGEMENT_QUEUE_
#define _MANAGEMENT_QUEUE_

//
// Licensed to the Apache Software Foundation (ASF) under one
// or more contributor license agreements.  See the NOTICE file
// distributed with this work for additional information
// regarding copyright ownership.  The ASF licenses this file
// to you under the Apache License, Version 2.0 (the
// "License"); you may not use this file except in compliance
// with the License.  You may obtain a copy of the License at
// 
//   http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing,
// software distributed under the License is distributed on an
// "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
// KIND, either express or implied.  See the License for the
// specific language governing permissions and limitations
// under the License.
//

// This source file was created by a code generator.
// Please do not edit.

#include "qpid/management/ManagementObject.h"
#include "qpid/framing/FieldTable.h"
#include "qpid/framing/Uuid.h"

namespace qpid {
namespace management {

class Queue : public ManagementObject
{
  private:

    static std::string packageName;
    static std::string className;
    static uint8_t     md5Sum[16];

    // Configuration Elements
    uint64_t vhostRef;
    std::string name;
    uint8_t durable;
    uint8_t autoDelete;
    uint8_t exclusive;
    framing::FieldTable arguments;
    uint64_t storeRef;

    // Instrumentation Elements
    uint64_t  msgTotalEnqueues;
    uint64_t  msgTotalDequeues;
    uint64_t  msgTxnEnqueues;
    uint64_t  msgTxnDequeues;
    uint64_t  msgPersistEnqueues;
    uint64_t  msgPersistDequeues;
    uint32_t  msgDepth;
    uint32_t  msgDepthHigh;
    uint32_t  msgDepthLow;
    uint64_t  byteTotalEnqueues;
    uint64_t  byteTotalDequeues;
    uint64_t  byteTxnEnqueues;
    uint64_t  byteTxnDequeues;
    uint64_t  bytePersistEnqueues;
    uint64_t  bytePersistDequeues;
    uint32_t  byteDepth;
    uint32_t  byteDepthHigh;
    uint32_t  byteDepthLow;
    uint64_t  enqueueTxnStarts;
    uint64_t  enqueueTxnCommits;
    uint64_t  enqueueTxnRejects;
    uint32_t  enqueueTxnCount;
    uint32_t  enqueueTxnCountHigh;
    uint32_t  enqueueTxnCountLow;
    uint64_t  dequeueTxnStarts;
    uint64_t  dequeueTxnCommits;
    uint64_t  dequeueTxnRejects;
    uint32_t  dequeueTxnCount;
    uint32_t  dequeueTxnCountHigh;
    uint32_t  dequeueTxnCountLow;
    uint32_t  consumers;
    uint32_t  consumersHigh;
    uint32_t  consumersLow;
    uint32_t  bindings;
    uint32_t  bindingsHigh;
    uint32_t  bindingsLow;
    uint32_t  unackedMessages;
    uint32_t  unackedMessagesHigh;
    uint32_t  unackedMessagesLow;
    uint64_t  messageLatencyCount;
    uint64_t  messageLatencyTotal;
    uint64_t  messageLatencyMin;
    uint64_t  messageLatencyMax;

    // Private Methods
    static void writeSchema   (qpid::framing::Buffer& buf);
    void writeConfig          (qpid::framing::Buffer& buf);
    void writeInstrumentation (qpid::framing::Buffer& buf,
                               bool skipHeaders = false);
    void doMethod             (std::string            methodName,
                               qpid::framing::Buffer& inBuf,
                               qpid::framing::Buffer& outBuf);
    writeSchemaCall_t getWriteSchemaCall (void) { return writeSchema; }


  public:

    friend class PackageQpid;
    typedef boost::shared_ptr<Queue> shared_ptr;

    Queue (Manageable* coreObject, Manageable* _parent, std::string _name, uint8_t _durable, uint8_t _autoDelete, uint8_t _exclusive);
    ~Queue (void);

    std::string getPackageName (void) { return packageName; }
    std::string getClassName   (void) { return className; }
    uint8_t*    getMd5Sum      (void) { return md5Sum; }

    // Method IDs
    static const uint32_t METHOD_PURGE = 1;

    // Accessor Methods
    inline void set_arguments (framing::FieldTable val){
        sys::RWlock::ScopedWlock writeLock (accessLock);
        arguments = val;
        configChanged = true;
    }
    inline void set_storeRef (uint64_t val){
        sys::RWlock::ScopedWlock writeLock (accessLock);
        storeRef = val;
        configChanged = true;
    }
    inline void inc_msgTotalEnqueues (uint64_t by = 1){
        sys::RWlock::ScopedWlock writeLock (accessLock);
        msgTotalEnqueues += by;
        instChanged = true;
    }
    inline void dec_msgTotalEnqueues (uint64_t by = 1){
        sys::RWlock::ScopedWlock writeLock (accessLock);
        msgTotalEnqueues -= by;
        instChanged = true;
    }
    inline void set_msgTotalEnqueues (uint64_t val){
        sys::RWlock::ScopedWlock writeLock (accessLock);
        msgTotalEnqueues = val;
        instChanged = true;
    }
    inline void inc_msgTotalDequeues (uint64_t by = 1){
        sys::RWlock::ScopedWlock writeLock (accessLock);
        msgTotalDequeues += by;
        instChanged = true;
    }
    inline void dec_msgTotalDequeues (uint64_t by = 1){
        sys::RWlock::ScopedWlock writeLock (accessLock);
        msgTotalDequeues -= by;
        instChanged = true;
    }
    inline void set_msgTotalDequeues (uint64_t val){
        sys::RWlock::ScopedWlock writeLock (accessLock);
        msgTotalDequeues = val;
        instChanged = true;
    }
    inline void inc_msgTxnEnqueues (uint64_t by = 1){
        sys::RWlock::ScopedWlock writeLock (accessLock);
        msgTxnEnqueues += by;
        instChanged = true;
    }
    inline void dec_msgTxnEnqueues (uint64_t by = 1){
        sys::RWlock::ScopedWlock writeLock (accessLock);
        msgTxnEnqueues -= by;
        instChanged = true;
    }
    inline void set_msgTxnEnqueues (uint64_t val){
        sys::RWlock::ScopedWlock writeLock (accessLock);
        msgTxnEnqueues = val;
        instChanged = true;
    }
    inline void inc_msgTxnDequeues (uint64_t by = 1){
        sys::RWlock::ScopedWlock writeLock (accessLock);
        msgTxnDequeues += by;
        instChanged = true;
    }
    inline void dec_msgTxnDequeues (uint64_t by = 1){
        sys::RWlock::ScopedWlock writeLock (accessLock);
        msgTxnDequeues -= by;
        instChanged = true;
    }
    inline void set_msgTxnDequeues (uint64_t val){
        sys::RWlock::ScopedWlock writeLock (accessLock);
        msgTxnDequeues = val;
        instChanged = true;
    }
    inline void inc_msgPersistEnqueues (uint64_t by = 1){
        sys::RWlock::ScopedWlock writeLock (accessLock);
        msgPersistEnqueues += by;
        instChanged = true;
    }
    inline void dec_msgPersistEnqueues (uint64_t by = 1){
        sys::RWlock::ScopedWlock writeLock (accessLock);
        msgPersistEnqueues -= by;
        instChanged = true;
    }
    inline void set_msgPersistEnqueues (uint64_t val){
        sys::RWlock::ScopedWlock writeLock (accessLock);
        msgPersistEnqueues = val;
        instChanged = true;
    }
    inline void inc_msgPersistDequeues (uint64_t by = 1){
        sys::RWlock::ScopedWlock writeLock (accessLock);
        msgPersistDequeues += by;
        instChanged = true;
    }
    inline void dec_msgPersistDequeues (uint64_t by = 1){
        sys::RWlock::ScopedWlock writeLock (accessLock);
        msgPersistDequeues -= by;
        instChanged = true;
    }
    inline void set_msgPersistDequeues (uint64_t val){
        sys::RWlock::ScopedWlock writeLock (accessLock);
        msgPersistDequeues = val;
        instChanged = true;
    }
    inline void inc_msgDepth (uint32_t by = 1){
        sys::RWlock::ScopedWlock writeLock (accessLock);
        msgDepth += by;
        if (msgDepthHigh < msgDepth)
            msgDepthHigh = msgDepth;
        instChanged = true;
    }
    inline void dec_msgDepth (uint32_t by = 1){
        sys::RWlock::ScopedWlock writeLock (accessLock);
        msgDepth -= by;
        if (msgDepthLow > msgDepth)
            msgDepthLow = msgDepth;
        instChanged = true;
    }
    inline void set_msgDepth (uint32_t val){
        sys::RWlock::ScopedWlock writeLock (accessLock);
        msgDepth = val;
        if (msgDepthLow  > val)
            msgDepthLow  = val;
        if (msgDepthHigh < val)
            msgDepthHigh = val;
        instChanged = true;
    }
    inline void inc_byteTotalEnqueues (uint64_t by = 1){
        sys::RWlock::ScopedWlock writeLock (accessLock);
        byteTotalEnqueues += by;
        instChanged = true;
    }
    inline void dec_byteTotalEnqueues (uint64_t by = 1){
        sys::RWlock::ScopedWlock writeLock (accessLock);
        byteTotalEnqueues -= by;
        instChanged = true;
    }
    inline void set_byteTotalEnqueues (uint64_t val){
        sys::RWlock::ScopedWlock writeLock (accessLock);
        byteTotalEnqueues = val;
        instChanged = true;
    }
    inline void inc_byteTotalDequeues (uint64_t by = 1){
        sys::RWlock::ScopedWlock writeLock (accessLock);
        byteTotalDequeues += by;
        instChanged = true;
    }
    inline void dec_byteTotalDequeues (uint64_t by = 1){
        sys::RWlock::ScopedWlock writeLock (accessLock);
        byteTotalDequeues -= by;
        instChanged = true;
    }
    inline void set_byteTotalDequeues (uint64_t val){
        sys::RWlock::ScopedWlock writeLock (accessLock);
        byteTotalDequeues = val;
        instChanged = true;
    }
    inline void inc_byteTxnEnqueues (uint64_t by = 1){
        sys::RWlock::ScopedWlock writeLock (accessLock);
        byteTxnEnqueues += by;
        instChanged = true;
    }
    inline void dec_byteTxnEnqueues (uint64_t by = 1){
        sys::RWlock::ScopedWlock writeLock (accessLock);
        byteTxnEnqueues -= by;
        instChanged = true;
    }
    inline void set_byteTxnEnqueues (uint64_t val){
        sys::RWlock::ScopedWlock writeLock (accessLock);
        byteTxnEnqueues = val;
        instChanged = true;
    }
    inline void inc_byteTxnDequeues (uint64_t by = 1){
        sys::RWlock::ScopedWlock writeLock (accessLock);
        byteTxnDequeues += by;
        instChanged = true;
    }
    inline void dec_byteTxnDequeues (uint64_t by = 1){
        sys::RWlock::ScopedWlock writeLock (accessLock);
        byteTxnDequeues -= by;
        instChanged = true;
    }
    inline void set_byteTxnDequeues (uint64_t val){
        sys::RWlock::ScopedWlock writeLock (accessLock);
        byteTxnDequeues = val;
        instChanged = true;
    }
    inline void inc_bytePersistEnqueues (uint64_t by = 1){
        sys::RWlock::ScopedWlock writeLock (accessLock);
        bytePersistEnqueues += by;
        instChanged = true;
    }
    inline void dec_bytePersistEnqueues (uint64_t by = 1){
        sys::RWlock::ScopedWlock writeLock (accessLock);
        bytePersistEnqueues -= by;
        instChanged = true;
    }
    inline void set_bytePersistEnqueues (uint64_t val){
        sys::RWlock::ScopedWlock writeLock (accessLock);
        bytePersistEnqueues = val;
        instChanged = true;
    }
    inline void inc_bytePersistDequeues (uint64_t by = 1){
        sys::RWlock::ScopedWlock writeLock (accessLock);
        bytePersistDequeues += by;
        instChanged = true;
    }
    inline void dec_bytePersistDequeues (uint64_t by = 1){
        sys::RWlock::ScopedWlock writeLock (accessLock);
        bytePersistDequeues -= by;
        instChanged = true;
    }
    inline void set_bytePersistDequeues (uint64_t val){
        sys::RWlock::ScopedWlock writeLock (accessLock);
        bytePersistDequeues = val;
        instChanged = true;
    }
    inline void inc_byteDepth (uint32_t by = 1){
        sys::RWlock::ScopedWlock writeLock (accessLock);
        byteDepth += by;
        if (byteDepthHigh < byteDepth)
            byteDepthHigh = byteDepth;
        instChanged = true;
    }
    inline void dec_byteDepth (uint32_t by = 1){
        sys::RWlock::ScopedWlock writeLock (accessLock);
        byteDepth -= by;
        if (byteDepthLow > byteDepth)
            byteDepthLow = byteDepth;
        instChanged = true;
    }
    inline void set_byteDepth (uint32_t val){
        sys::RWlock::ScopedWlock writeLock (accessLock);
        byteDepth = val;
        if (byteDepthLow  > val)
            byteDepthLow  = val;
        if (byteDepthHigh < val)
            byteDepthHigh = val;
        instChanged = true;
    }
    inline void inc_enqueueTxnStarts (uint64_t by = 1){
        sys::RWlock::ScopedWlock writeLock (accessLock);
        enqueueTxnStarts += by;
        instChanged = true;
    }
    inline void dec_enqueueTxnStarts (uint64_t by = 1){
        sys::RWlock::ScopedWlock writeLock (accessLock);
        enqueueTxnStarts -= by;
        instChanged = true;
    }
    inline void set_enqueueTxnStarts (uint64_t val){
        sys::RWlock::ScopedWlock writeLock (accessLock);
        enqueueTxnStarts = val;
        instChanged = true;
    }
    inline void inc_enqueueTxnCommits (uint64_t by = 1){
        sys::RWlock::ScopedWlock writeLock (accessLock);
        enqueueTxnCommits += by;
        instChanged = true;
    }
    inline void dec_enqueueTxnCommits (uint64_t by = 1){
        sys::RWlock::ScopedWlock writeLock (accessLock);
        enqueueTxnCommits -= by;
        instChanged = true;
    }
    inline void set_enqueueTxnCommits (uint64_t val){
        sys::RWlock::ScopedWlock writeLock (accessLock);
        enqueueTxnCommits = val;
        instChanged = true;
    }
    inline void inc_enqueueTxnRejects (uint64_t by = 1){
        sys::RWlock::ScopedWlock writeLock (accessLock);
        enqueueTxnRejects += by;
        instChanged = true;
    }
    inline void dec_enqueueTxnRejects (uint64_t by = 1){
        sys::RWlock::ScopedWlock writeLock (accessLock);
        enqueueTxnRejects -= by;
        instChanged = true;
    }
    inline void set_enqueueTxnRejects (uint64_t val){
        sys::RWlock::ScopedWlock writeLock (accessLock);
        enqueueTxnRejects = val;
        instChanged = true;
    }
    inline void inc_enqueueTxnCount (uint32_t by = 1){
        sys::RWlock::ScopedWlock writeLock (accessLock);
        enqueueTxnCount += by;
        if (enqueueTxnCountHigh < enqueueTxnCount)
            enqueueTxnCountHigh = enqueueTxnCount;
        instChanged = true;
    }
    inline void dec_enqueueTxnCount (uint32_t by = 1){
        sys::RWlock::ScopedWlock writeLock (accessLock);
        enqueueTxnCount -= by;
        if (enqueueTxnCountLow > enqueueTxnCount)
            enqueueTxnCountLow = enqueueTxnCount;
        instChanged = true;
    }
    inline void set_enqueueTxnCount (uint32_t val){
        sys::RWlock::ScopedWlock writeLock (accessLock);
        enqueueTxnCount = val;
        if (enqueueTxnCountLow  > val)
            enqueueTxnCountLow  = val;
        if (enqueueTxnCountHigh < val)
            enqueueTxnCountHigh = val;
        instChanged = true;
    }
    inline void inc_dequeueTxnStarts (uint64_t by = 1){
        sys::RWlock::ScopedWlock writeLock (accessLock);
        dequeueTxnStarts += by;
        instChanged = true;
    }
    inline void dec_dequeueTxnStarts (uint64_t by = 1){
        sys::RWlock::ScopedWlock writeLock (accessLock);
        dequeueTxnStarts -= by;
        instChanged = true;
    }
    inline void set_dequeueTxnStarts (uint64_t val){
        sys::RWlock::ScopedWlock writeLock (accessLock);
        dequeueTxnStarts = val;
        instChanged = true;
    }
    inline void inc_dequeueTxnCommits (uint64_t by = 1){
        sys::RWlock::ScopedWlock writeLock (accessLock);
        dequeueTxnCommits += by;
        instChanged = true;
    }
    inline void dec_dequeueTxnCommits (uint64_t by = 1){
        sys::RWlock::ScopedWlock writeLock (accessLock);
        dequeueTxnCommits -= by;
        instChanged = true;
    }
    inline void set_dequeueTxnCommits (uint64_t val){
        sys::RWlock::ScopedWlock writeLock (accessLock);
        dequeueTxnCommits = val;
        instChanged = true;
    }
    inline void inc_dequeueTxnRejects (uint64_t by = 1){
        sys::RWlock::ScopedWlock writeLock (accessLock);
        dequeueTxnRejects += by;
        instChanged = true;
    }
    inline void dec_dequeueTxnRejects (uint64_t by = 1){
        sys::RWlock::ScopedWlock writeLock (accessLock);
        dequeueTxnRejects -= by;
        instChanged = true;
    }
    inline void set_dequeueTxnRejects (uint64_t val){
        sys::RWlock::ScopedWlock writeLock (accessLock);
        dequeueTxnRejects = val;
        instChanged = true;
    }
    inline void inc_dequeueTxnCount (uint32_t by = 1){
        sys::RWlock::ScopedWlock writeLock (accessLock);
        dequeueTxnCount += by;
        if (dequeueTxnCountHigh < dequeueTxnCount)
            dequeueTxnCountHigh = dequeueTxnCount;
        instChanged = true;
    }
    inline void dec_dequeueTxnCount (uint32_t by = 1){
        sys::RWlock::ScopedWlock writeLock (accessLock);
        dequeueTxnCount -= by;
        if (dequeueTxnCountLow > dequeueTxnCount)
            dequeueTxnCountLow = dequeueTxnCount;
        instChanged = true;
    }
    inline void set_dequeueTxnCount (uint32_t val){
        sys::RWlock::ScopedWlock writeLock (accessLock);
        dequeueTxnCount = val;
        if (dequeueTxnCountLow  > val)
            dequeueTxnCountLow  = val;
        if (dequeueTxnCountHigh < val)
            dequeueTxnCountHigh = val;
        instChanged = true;
    }
    inline void inc_consumers (uint32_t by = 1){
        sys::RWlock::ScopedWlock writeLock (accessLock);
        consumers += by;
        if (consumersHigh < consumers)
            consumersHigh = consumers;
        instChanged = true;
    }
    inline void dec_consumers (uint32_t by = 1){
        sys::RWlock::ScopedWlock writeLock (accessLock);
        consumers -= by;
        if (consumersLow > consumers)
            consumersLow = consumers;
        instChanged = true;
    }
    inline void set_consumers (uint32_t val){
        sys::RWlock::ScopedWlock writeLock (accessLock);
        consumers = val;
        if (consumersLow  > val)
            consumersLow  = val;
        if (consumersHigh < val)
            consumersHigh = val;
        instChanged = true;
    }
    inline void inc_bindings (uint32_t by = 1){
        sys::RWlock::ScopedWlock writeLock (accessLock);
        bindings += by;
        if (bindingsHigh < bindings)
            bindingsHigh = bindings;
        instChanged = true;
    }
    inline void dec_bindings (uint32_t by = 1){
        sys::RWlock::ScopedWlock writeLock (accessLock);
        bindings -= by;
        if (bindingsLow > bindings)
            bindingsLow = bindings;
        instChanged = true;
    }
    inline void set_bindings (uint32_t val){
        sys::RWlock::ScopedWlock writeLock (accessLock);
        bindings = val;
        if (bindingsLow  > val)
            bindingsLow  = val;
        if (bindingsHigh < val)
            bindingsHigh = val;
        instChanged = true;
    }
    inline void inc_unackedMessages (uint32_t by = 1){
        sys::RWlock::ScopedWlock writeLock (accessLock);
        unackedMessages += by;
        if (unackedMessagesHigh < unackedMessages)
            unackedMessagesHigh = unackedMessages;
        instChanged = true;
    }
    inline void dec_unackedMessages (uint32_t by = 1){
        sys::RWlock::ScopedWlock writeLock (accessLock);
        unackedMessages -= by;
        if (unackedMessagesLow > unackedMessages)
            unackedMessagesLow = unackedMessages;
        instChanged = true;
    }
    inline void set_unackedMessages (uint32_t val){
        sys::RWlock::ScopedWlock writeLock (accessLock);
        unackedMessages = val;
        if (unackedMessagesLow  > val)
            unackedMessagesLow  = val;
        if (unackedMessagesHigh < val)
            unackedMessagesHigh = val;
        instChanged = true;
    }
    inline void set_messageLatency (uint64_t val){
        sys::RWlock::ScopedWlock writeLock (accessLock);
        messageLatencyCount++;
        messageLatencyTotal += val;
        if (messageLatencyMin > val)
            messageLatencyMin = val;
        if (messageLatencyMax < val)
            messageLatencyMax = val;
        instChanged = true;
    }

};

}}
            

#endif  /*!_MANAGEMENT_QUEUE_*/
