
//
// Licensed to the Apache Software Foundation (ASF) under one
// or more contributor license agreements.  See the NOTICE file
// distributed with this work for additional information
// regarding copyright ownership.  The ASF licenses this file
// to you under the Apache License, Version 2.0 (the
// "License"); you may not use this file except in compliance
// with the License.  You may obtain a copy of the License at
// 
//   http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing,
// software distributed under the License is distributed on an
// "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
// KIND, either express or implied.  See the License for the
// specific language governing permissions and limitations
// under the License.
//

// This source file was created by a code generator.
// Please do not edit.

#include "qpid/log/Statement.h"
#include "qpid/framing/FieldTable.h"
#include "qpid/management/Manageable.h" 
#include "Session.h"


using namespace qpid::management;
using namespace qpid::sys;
using namespace qpid::framing;
using           std::string;

string  Session::packageName  = string ("qpid");
string  Session::className    = string ("session");
uint8_t Session::md5Sum[16]   =
    {0x69,0xda,0x6e,0xe1,0x6f,0xa,0xfb,0xc5,0x6a,0x22,0x9f,0xd,0xb4,0xd7,0xc7,0xe9};

Session::Session (Manageable* _core, Manageable* _parent, std::string _name) :
    ManagementObject(_core)
    ,name(_name)
{
    vhostRef = _parent->GetManagementObject ()->getObjectId ();
    attached = 0;
    expireTime = 0;
    framesOutstanding = 0;

}

Session::~Session () {}

namespace {
    const string NAME("name");
    const string TYPE("type");
    const string ACCESS("access");
    const string INDEX("index");
    const string UNIT("unit");
    const string MIN("min");
    const string MAX("max");
    const string MAXLEN("maxlen");
    const string DESC("desc");
    const string ARGCOUNT("argCount");
    const string ARGS("args");
    const string DIR("dir");
    const string DEFAULT("default");
}

void Session::writeSchema (Buffer& buf)
{
    FieldTable ft;

    // Schema class header:
    buf.putShortString (packageName); // Package Name
    buf.putShortString (className);   // Class Name
    buf.putBin128      (md5Sum);      // Schema Hash
    buf.putShort       (5); // Config Element Count
    buf.putShort       (3); // Inst Element Count
    buf.putShort       (4); // Method Count
    buf.putShort       (0); // Event Count

    // Config Elements
    ft = FieldTable ();
    ft.setString (NAME,   "vhostRef");
    ft.setInt    (TYPE,   TYPE_REF);
    ft.setInt    (ACCESS, ACCESS_RC);
    ft.setInt    (INDEX,  1);
    buf.put (ft);

    ft = FieldTable ();
    ft.setString (NAME,   "name");
    ft.setInt    (TYPE,   TYPE_SSTR);
    ft.setInt    (ACCESS, ACCESS_RC);
    ft.setInt    (INDEX,  1);
    buf.put (ft);

    ft = FieldTable ();
    ft.setString (NAME,   "channelId");
    ft.setInt    (TYPE,   TYPE_U16);
    ft.setInt    (ACCESS, ACCESS_RO);
    ft.setInt    (INDEX,  0);
    buf.put (ft);

    ft = FieldTable ();
    ft.setString (NAME,   "clientRef");
    ft.setInt    (TYPE,   TYPE_REF);
    ft.setInt    (ACCESS, ACCESS_RO);
    ft.setInt    (INDEX,  0);
    buf.put (ft);

    ft = FieldTable ();
    ft.setString (NAME,   "detachedLifespan");
    ft.setInt    (TYPE,   TYPE_U32);
    ft.setInt    (ACCESS, ACCESS_RO);
    ft.setInt    (INDEX,  0);
    ft.setString (UNIT,   "second");
    buf.put (ft);


    // Inst Elements
    ft = FieldTable ();
    ft.setString (NAME,   "attached");
    ft.setInt    (TYPE,   TYPE_BOOL);
    buf.put (ft);

    ft = FieldTable ();
    ft.setString (NAME,   "expireTime");
    ft.setInt    (TYPE,   TYPE_ABSTIME);
    buf.put (ft);

    ft = FieldTable ();
    ft.setString (NAME,   "framesOutstanding");
    ft.setInt    (TYPE,   TYPE_U32);
    buf.put (ft);


    // Methods
    ft = FieldTable ();
    ft.setString (NAME,     "solicitAck");
    ft.setInt    (ARGCOUNT, 0);
    buf.put (ft);

    ft = FieldTable ();
    ft.setString (NAME,     "detach");
    ft.setInt    (ARGCOUNT, 0);
    buf.put (ft);

    ft = FieldTable ();
    ft.setString (NAME,     "resetLifespan");
    ft.setInt    (ARGCOUNT, 0);
    buf.put (ft);

    ft = FieldTable ();
    ft.setString (NAME,     "close");
    ft.setInt    (ARGCOUNT, 0);
    buf.put (ft);


    // Events

}

void Session::writeConfig (Buffer& buf)
{
    sys::RWlock::ScopedRlock readLock (accessLock);
    configChanged = false;

    writeTimestamps (buf);
    buf.putLongLong (vhostRef);
    buf.putShortString (name);
    buf.putShort (channelId);
    buf.putLongLong (clientRef);
    buf.putLong (detachedLifespan);

}

void Session::writeInstrumentation (Buffer& buf, bool skipHeaders)
{
    sys::RWlock::ScopedWlock writeLock (accessLock);
    instChanged = false;

    if (!skipHeaders)
        writeTimestamps (buf);
    buf.putOctet (attached?1:0);
    buf.putLongLong (expireTime);
    buf.putLong (framesOutstanding);


    // Maintenance of hi-lo statistics

}

void Session::doMethod (string methodName, Buffer&, Buffer& outBuf)
{
    Manageable::status_t status = Manageable::STATUS_UNKNOWN_METHOD;

    if (methodName == "solicitAck")
    {
        ArgsNone ioArgs;
        status = coreObject->ManagementMethod (METHOD_SOLICITACK, ioArgs);
        outBuf.putLong        (status);
        outBuf.putShortString (Manageable::StatusText (status));
        return;
    }

    if (methodName == "detach")
    {
        ArgsNone ioArgs;
        status = coreObject->ManagementMethod (METHOD_DETACH, ioArgs);
        outBuf.putLong        (status);
        outBuf.putShortString (Manageable::StatusText (status));
        return;
    }

    if (methodName == "resetLifespan")
    {
        ArgsNone ioArgs;
        status = coreObject->ManagementMethod (METHOD_RESETLIFESPAN, ioArgs);
        outBuf.putLong        (status);
        outBuf.putShortString (Manageable::StatusText (status));
        return;
    }

    if (methodName == "close")
    {
        ArgsNone ioArgs;
        status = coreObject->ManagementMethod (METHOD_CLOSE, ioArgs);
        outBuf.putLong        (status);
        outBuf.putShortString (Manageable::StatusText (status));
        return;
    }

    outBuf.putLong        (status);
    outBuf.putShortString (Manageable::StatusText (status));
}

